=== modified file 'BRANCH.TODO'
--- BRANCH.TODO	2006-06-03 00:10:32 +0000
+++ BRANCH.TODO	2006-06-03 16:23:09 +0000
@@ -1,3 +1,5 @@
 # This file is for listing TODOs for branches that are being worked on.
 # It should ALWAYS be empty in the mainline or in integration branches.
 # 
+
+Test osutils.file_kind for more than just regular files and directories

=== modified file 'NEWS'
--- NEWS	2006-05-24 08:14:45 +0000
+++ NEWS	2006-05-30 15:33:52 +0000
@@ -56,6 +56,9 @@
       all of the history in the repository. This is especially important for
       shared repositories. (John Arbash Meinel)
 
+    * bzrlib.delta.compare_trees now iterates in alphabetically sorted order,
+      rather than randomly walking the inventories. (John Arbash Meinel)
+
 bzr 0.8.2  2006-05-17
   
   BUG FIXES:

=== modified file 'TODO'
--- TODO	2006-06-03 00:10:32 +0000
+++ TODO	2006-06-03 16:23:09 +0000
@@ -184,6 +184,10 @@
 
 * BZRDIR should be in branch.py not __init__.py.
 
+* bzrlib.delta.compare_trees() shouldn't need to sort all of the result
+  lists, since they are being added in alphabetical order. Write tests
+  to guarantee this, and remove the sort() calls.
+
 Medium things
 -------------
 

=== modified file 'bzrlib/benchmarks/__init__.py'
--- bzrlib/benchmarks/__init__.py	2006-05-25 04:21:23 +0000
+++ bzrlib/benchmarks/__init__.py	2006-05-27 06:18:34 +0000
@@ -49,5 +49,6 @@
                    'bzrlib.benchmarks.bench_commit',
                    'bzrlib.benchmarks.bench_status',
                    'bzrlib.benchmarks.bench_transform',
+                   'bzrlib.benchmarks.bench_workingtree',
                    ]
     return TestLoader().loadTestsFromModuleNames(testmod_names)

=== modified file 'bzrlib/benchmarks/bench_status.py'
--- bzrlib/benchmarks/bench_status.py	2006-05-22 09:17:55 +0000
+++ bzrlib/benchmarks/bench_status.py	2006-05-27 03:11:07 +0000
@@ -29,3 +29,10 @@
         self.run_bzr('add')
         # on robertc's machine the first sample of this took 1687ms/15994ms
         self.time(self.run_bzr, 'status')
+
+    def test_no_changes_known_kernel_like_tree(self):
+        """Status in a kernel sized tree with no ignored, unknowns, or added.""" 
+        self.make_kernel_like_tree()
+        self.run_bzr('add')
+        self.run_bzr('commit', '-m', 'initial import')
+        self.time(self.run_bzr, 'status')

=== added file 'bzrlib/benchmarks/bench_workingtree.py'
--- bzrlib/benchmarks/bench_workingtree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/benchmarks/bench_workingtree.py	2006-05-30 04:33:16 +0000
@@ -0,0 +1,45 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for bzr working tree performance."""
+
+import os
+
+from bzrlib.benchmarks import Benchmark
+from bzrlib.workingtree import WorkingTree
+
+
+class WorkingTreeBenchmark(Benchmark):
+
+    def test_list_files_kernel_like_tree(self):
+        self.make_kernel_like_tree()
+        self.run_bzr('add')
+        tree = WorkingTree.open('.')
+        self.time(list, tree.list_files())
+
+    def test_list_files_unknown_kernel_like_tree(self):
+        self.make_kernel_like_tree()
+        tree = WorkingTree.open('.')
+        # Bzr only traverses directories if they are versioned
+        # So add all the directories, but not the files, yielding
+        # lots of unknown files.
+        for root, dirs, files in os.walk('.'):
+            if '.bzr' in dirs:
+                dirs.remove('.bzr')
+            if root == '.':
+                continue
+            tree.add(root)
+        self.time(list, tree.list_files())
+

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-05-30 15:53:33 +0000
+++ bzrlib/builtins.py	2006-06-03 16:23:09 +0000
@@ -795,13 +795,29 @@
 
     This makes bzr stop tracking changes to a versioned file.  It does
     not delete the working copy.
+
+    You can specify one or more files, and/or --new.  If you specify --new,
+    only 'added' files will be removed.  If you specify both, then new files
+    in the specified directories will be removed.  If the directories are
+    also new, they will also be removed.
     """
-    takes_args = ['file+']
-    takes_options = ['verbose']
+    takes_args = ['file*']
+    takes_options = ['verbose', Option('new', help='remove newly-added files')]
     aliases = ['rm']
     
-    def run(self, file_list, verbose=False):
+    def run(self, file_list, verbose=False, new=False):
         tree, file_list = tree_files(file_list)
+        if new is False:
+            if file_list is None:
+                raise BzrCommandError('Specify one or more files to remove, or'
+                                      ' use --new.')
+        else:
+            from bzrlib.delta import compare_trees
+            added = [compare_trees(tree.basis_tree(), tree,
+                                   specific_files=file_list).added]
+            file_list = sorted([f[0] for f in added[0]], reverse=True)
+            if len(file_list) == 0:
+                raise BzrCommandError('No matching files.')
         tree.remove(file_list, verbose=verbose)
 
 
@@ -1860,6 +1876,18 @@
         print "  nick: %s" % (branch.nick,)
         if rh:
             print "  revid: %s" % (rh[-1],)
+    print "Using python interpreter:", sys.executable
+    import site
+    print "Using python standard library:", os.path.dirname(site.__file__)
+    print "Using bzrlib:",
+    if len(bzrlib.__path__) > 1:
+        # print repr, which is a good enough way of making it clear it's
+        # more than one element (eg ['/foo/bar', '/foo/bzr'])
+        print repr(bzrlib.__path__)
+    else:
+        print bzrlib.__path__[0]
+
+    print
     print bzrlib.__copyright__
     print "http://bazaar-vcs.org/"
     print

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-05-19 07:57:09 +0000
+++ bzrlib/bzrdir.py	2006-05-28 03:56:21 +0000
@@ -47,7 +47,7 @@
 from bzrlib.transport.local import LocalTransport
 from bzrlib.weave import Weave
 from bzrlib.xml4 import serializer_v4
-from bzrlib.xml5 import serializer_v5
+import bzrlib.xml5
 
 
 class BzrDir(object):
@@ -1464,7 +1464,7 @@
 
     def _convert_working_inv(self):
         inv = serializer_v4.read_inventory(self.branch.control_files.get('inventory'))
-        new_inv_xml = serializer_v5.write_inventory_to_string(inv)
+        new_inv_xml = bzrlib.xml5.serializer_v5.write_inventory_to_string(inv)
         # FIXME inventory is a working tree change.
         self.branch.control_files.put('inventory', new_inv_xml)
 
@@ -1538,7 +1538,7 @@
     def _load_updated_inventory(self, rev_id):
         assert rev_id in self.converted_revs
         inv_xml = self.inv_weave.get_text(rev_id)
-        inv = serializer_v5.read_inventory_from_string(inv_xml)
+        inv = bzrlib.xml5.serializer_v5.read_inventory_from_string(inv_xml)
         return inv
 
     def _convert_one_rev(self, rev_id):
@@ -1561,7 +1561,7 @@
                 assert hasattr(ie, 'revision'), \
                     'no revision on {%s} in {%s}' % \
                     (file_id, rev.revision_id)
-        new_inv_xml = serializer_v5.write_inventory_to_string(inv)
+        new_inv_xml = bzrlib.xml5.serializer_v5.write_inventory_to_string(inv)
         new_inv_sha1 = sha_string(new_inv_xml)
         self.inv_weave.add_lines(rev.revision_id, 
                                  present_parents,

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2006-04-19 07:16:05 +0000
+++ bzrlib/delta.py	2006-05-30 04:44:48 +0000
@@ -17,6 +17,7 @@
 from bzrlib.inventory import InventoryEntry
 from bzrlib.trace import mutter
 
+
 class TreeDelta(object):
     """Describes changes from one tree to another.
 
@@ -188,75 +189,146 @@
     # Perhaps should take a list of file-ids instead?   Need to indicate any
     # ids or names which were not found in the trees.
 
-    for file_id in old_tree:
-        if file_id in new_tree:
-            old_ie = old_inv[file_id]
-            new_ie = new_inv[file_id]
-
-            kind = old_ie.kind
-            assert kind == new_ie.kind
-            
-            assert kind in InventoryEntry.known_kinds, \
-                   'invalid file kind %r' % kind
-
-            if kind == 'root_directory':
-                continue
-            
-            if specific_files:
-                if (not is_inside_any(specific_files, old_inv.id2path(file_id)) 
-                    and not is_inside_any(specific_files, new_inv.id2path(file_id))):
-                    continue
-
-            # temporary hack until all entries are populated before clients 
-            # get them
-            old_path = old_inv.id2path(file_id)
-            new_path = new_inv.id2path(file_id)
-            old_ie._read_tree_state(old_path, old_tree)
-            new_ie._read_tree_state(new_path, new_tree)
-            text_modified, meta_modified = new_ie.detect_changes(old_ie)
-
-            # TODO: Can possibly avoid calculating path strings if the
-            # two files are unchanged and their names and parents are
-            # the same and the parents are unchanged all the way up.
-            # May not be worthwhile.
-            
-            if (old_ie.name != new_ie.name
-                or old_ie.parent_id != new_ie.parent_id):
-                delta.renamed.append((old_path,
-                                      new_path,
-                                      file_id, kind,
-                                      text_modified, meta_modified))
-            elif text_modified or meta_modified:
-                delta.modified.append((new_path, file_id, kind,
-                                       text_modified, meta_modified))
-            elif want_unchanged:
-                delta.unchanged.append((new_path, file_id, kind))
-        else:
-            kind = old_inv.get_file_kind(file_id)
-            if kind == 'root_directory':
-                continue
-            old_path = old_inv.id2path(file_id)
-            if specific_files:
-                if not is_inside_any(specific_files, old_path):
-                    continue
-            delta.removed.append((old_path, file_id, kind))
-
-    mutter('start looking for new files')
-    for file_id in new_inv:
-        if file_id in old_inv or file_id not in new_tree:
-            continue
-        kind = new_inv.get_file_kind(file_id)
-        if kind == 'root_directory':
-            continue
-        new_path = new_inv.id2path(file_id)
+    old_files = old_tree.list_files()
+    new_files = new_tree.list_files()
+
+    more_old = True
+    more_new = True
+
+    added = {}
+    removed = {}
+
+    def get_next(iter):
+        try:
+            return iter.next()
+        except StopIteration:
+            return None, None, None, None, None
+    old_path, old_class, old_kind, old_file_id, old_entry = get_next(old_files)
+    new_path, new_class, new_kind, new_file_id, new_entry = get_next(new_files)
+
+
+    def check_matching(old_path, old_entry, new_path, new_entry):
+        """We have matched up 2 file_ids, check for changes."""
+        assert old_entry.kind == new_entry.kind
+
+        if old_entry.kind == 'root_directory':
+            return
+
+        if specific_files:
+            if (not is_inside_any(specific_files, old_path)
+                and not is_inside_any(specific_files, new_path)):
+                return
+
+        # temporary hack until all entries are populated before clients 
+        # get them
+        old_entry._read_tree_state(old_path, old_tree)
+        new_entry._read_tree_state(new_path, new_tree)
+        text_modified, meta_modified = new_entry.detect_changes(old_entry)
+        
+        # If the name changes, or the parent_id changes, we have a rename
+        # (if we move a parent, that doesn't count as a rename for the file)
+        if (old_entry.name != new_entry.name 
+            or old_entry.parent_id != new_entry.parent_id):
+            delta.renamed.append((old_path,
+                                  new_path,
+                                  old_entry.file_id, old_entry.kind,
+                                  text_modified, meta_modified))
+        elif text_modified or meta_modified:
+            delta.modified.append((new_path, new_entry.file_id, new_entry.kind,
+                                   text_modified, meta_modified))
+        elif want_unchanged:
+            delta.unchanged.append((new_path, new_entry.file_id, new_entry.kind))
+
+
+    def handle_old(path, entry):
+        """old entry without a new entry match
+
+        Check to see if a matching new entry was already seen as an
+        added file, and switch the pair into being a rename.
+        Otherwise just mark the old entry being removed.
+        """
+        if entry.file_id in added:
+            # Actually this is a rename, we found a new file_id earlier
+            # at a different location, so it is no-longer added
+            x_new_path, x_new_entry = added.pop(entry.file_id)
+            check_matching(path, entry, x_new_path, x_new_entry)
+        else:
+            # We have an old_file_id which doesn't line up with a new_file_id
+            # So this file looks to be removed
+            assert entry.file_id not in removed
+            removed[entry.file_id] = path, entry
+
+    def handle_new(path, entry):
+        """new entry without an old entry match
+        
+        Check to see if a matching old entry was already seen as a
+        removal, and change the pair into a rename.
+        Otherwise just mark the new entry as an added file.
+        """
+        if entry.file_id in removed:
+            # We saw this file_id earlier at an old different location
+            # it is no longer removed, just renamed
+            x_old_path, x_old_entry = removed.pop(entry.file_id)
+            check_matching(x_old_path, x_old_entry, path, entry)
+        else:
+            # We have a new file which does not match an old file
+            # mark it as added
+            assert entry.file_id not in added
+            added[entry.file_id] = path, entry
+
+    while old_path or new_path:
+        # list_files() returns files in alphabetical path sorted order
+        if old_path == new_path:
+            if old_file_id == new_file_id:
+                # This is the common case, the files are in the same place
+                # check if there were any content changes
+
+                if old_file_id is None:
+                    # We have 2 unversioned files, no deltas possible???
+                    pass
+                else:
+                    check_matching(old_path, old_entry, new_path, new_entry)
+            else:
+                # The ids don't match, so we have to handle them both
+                # separately.
+                if old_file_id is not None:
+                    handle_old(old_path, old_entry)
+
+                if new_file_id is not None:
+                    handle_new(new_path, new_entry)
+
+            # The two entries were at the same path, so increment both sides
+            old_path, old_class, old_kind, old_file_id, old_entry = get_next(old_files)
+            new_path, new_class, new_kind, new_file_id, new_entry = get_next(new_files)
+        elif new_path is None or (old_path is not None and old_path < new_path):
+            # Assume we don't match, only process old_path
+            if old_file_id is not None:
+                handle_old(old_path, old_entry)
+            # old_path came first, so increment it, trying to match up
+            old_path, old_class, old_kind, old_file_id, old_entry = get_next(old_files)
+        elif new_path is not None:
+            # new_path came first, so increment it, trying to match up
+            if new_file_id is not None:
+                handle_new(new_path, new_entry)
+            new_path, new_class, new_kind, new_file_id, new_entry = get_next(new_files)
+
+    # Now we have a set of added and removed files, mark them all
+    for old_path, old_entry in removed.itervalues():
+        if specific_files:
+            if not is_inside_any(specific_files, old_path):
+                continue
+        delta.removed.append((old_path, old_entry.file_id, old_entry.kind))
+    for new_path, new_entry in added.itervalues():
         if specific_files:
             if not is_inside_any(specific_files, new_path):
                 continue
-        delta.added.append((new_path, file_id, kind))
-            
+        delta.added.append((new_path, new_entry.file_id, new_entry.kind))
+
     delta.removed.sort()
     delta.added.sort()
     delta.renamed.sort()
+    # TODO: jam 20060529 These lists shouldn't need to be sorted
+    #       since we added them in alphabetical order.
     delta.modified.sort()
     delta.unchanged.sort()
 

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2006-05-29 01:00:30 +0000
+++ bzrlib/diff.py	2006-05-31 04:12:42 +0000
@@ -290,12 +290,12 @@
     has_changes = 0
     for path, file_id, kind in delta.removed:
         has_changes = 1
-        print >>to_file, '=== removed %s %r' % (kind, path)
+        print >>to_file, '=== removed %s %r' % (kind, path.encode('utf8'))
         old_tree.inventory[file_id].diff(diff_file, old_label + path, old_tree,
                                          DEVNULL, None, None, to_file)
     for path, file_id, kind in delta.added:
         has_changes = 1
-        print >>to_file, '=== added %s %r' % (kind, path)
+        print >>to_file, '=== added %s %r' % (kind, path.encode('utf8'))
         new_tree.inventory[file_id].diff(diff_file, new_label + path, new_tree,
                                          DEVNULL, None, None, to_file, 
                                          reverse=True)
@@ -304,14 +304,15 @@
         has_changes = 1
         prop_str = get_prop_change(meta_modified)
         print >>to_file, '=== renamed %s %r => %r%s' % (
-                    kind, old_path, new_path, prop_str)
+                    kind, old_path.encode('utf8'),
+                    new_path.encode('utf8'), prop_str)
         _maybe_diff_file_or_symlink(old_label, old_path, old_tree, file_id,
                                     new_label, new_path, new_tree,
                                     text_modified, kind, to_file, diff_file)
     for path, file_id, kind, text_modified, meta_modified in delta.modified:
         has_changes = 1
         prop_str = get_prop_change(meta_modified)
-        print >>to_file, '=== modified %s %r%s' % (kind, path, prop_str)
+        print >>to_file, '=== modified %s %r%s' % (kind, path.encode('utf8'), prop_str)
         if text_modified:
             _maybe_diff_file_or_symlink(old_label, path, old_tree, file_id,
                                         new_label, path, new_tree,

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2006-05-19 13:53:22 +0000
+++ bzrlib/inventory.py	2006-05-27 09:24:05 +0000
@@ -28,6 +28,7 @@
 ROOT_ID = "TREE_ROOT"
 
 
+import collections
 import os.path
 import re
 import sys
@@ -668,8 +669,8 @@
 
     def _read_tree_state(self, path, work_tree):
         """See InventoryEntry._read_tree_state."""
-        self.text_sha1 = work_tree.get_file_sha1(self.file_id)
-        self.executable = work_tree.is_executable(self.file_id)
+        self.text_sha1 = work_tree.get_file_sha1(self.file_id, path=path)
+        self.executable = work_tree.is_executable(self.file_id, path=path)
 
     def _forget_tree_state(self):
         self.text_sha1 = None
@@ -827,7 +828,7 @@
     May also look up by name:
 
     >>> [x[0] for x in inv.iter_entries()]
-    ['hello.c']
+    [u'hello.c']
     >>> inv = Inventory('TREE_ROOT-12345678-12345678')
     >>> inv.add(InventoryFile('123-123', 'hello.c', ROOT_ID))
     InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT-12345678-12345678')
@@ -880,14 +881,39 @@
         elif isinstance(from_dir, basestring):
             from_dir = self._byid[from_dir]
             
-        kids = from_dir.children.items()
-        kids.sort()
-        for name, ie in kids:
-            yield name, ie
-            if ie.kind == 'directory':
-                for cn, cie in self.iter_entries(from_dir=ie.file_id):
-                    yield pathjoin(name, cn), cie
-
+        # unrolling the recursive called changed the time from
+        # 440ms/663ms (inline/total) to 116ms/116ms
+        children = from_dir.children.items()
+        children.sort()
+        children = collections.deque(children)
+        stack = [(u'', children)]
+        while stack:
+            from_dir_relpath, children = stack[-1]
+
+            while children:
+                name, ie = children.popleft()
+
+                # we know that from_dir_relpath never ends in a slash
+                # and 'f' doesn't begin with one, we can do a string op, rather
+                # than the checks of pathjoin(), though this means that all paths
+                # start with a slash
+                path = from_dir_relpath + '/' + name
+
+                yield path[1:], ie
+
+                if ie.kind != 'directory':
+                    continue
+
+                # But do this child first
+                new_children = ie.children.items()
+                new_children.sort()
+                new_children = collections.deque(new_children)
+                stack.append((path, new_children))
+                # Break out of inner loop, so that we start outer loop with child
+                break
+            else:
+                # if we finished all children, pop it off the stack
+                stack.pop()
 
     def entries(self):
         """Return list of (path, ie) for all entries except the root.

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2006-05-09 07:16:38 +0000
+++ bzrlib/osutils.py	2006-05-30 05:34:57 +0000
@@ -25,6 +25,7 @@
 import re
 import sha
 import shutil
+import stat
 import string
 import sys
 import time
@@ -38,6 +39,7 @@
                            PathNotChild,
                            IllegalPath,
                            )
+from bzrlib.symbol_versioning import *
 from bzrlib.trace import mutter
 import bzrlib.win32console
 
@@ -74,24 +76,20 @@
         return f
 
 
-def file_kind(f):
-    mode = os.lstat(f)[ST_MODE]
-    if S_ISREG(mode):
-        return 'file'
-    elif S_ISDIR(mode):
-        return 'directory'
-    elif S_ISLNK(mode):
-        return 'symlink'
-    elif S_ISCHR(mode):
-        return 'chardev'
-    elif S_ISBLK(mode):
-        return 'block'
-    elif S_ISFIFO(mode):
-        return 'fifo'
-    elif S_ISSOCK(mode):
-        return 'socket'
-    else:
-        return 'unknown'
+_formats = {
+    stat.S_IFDIR:'directory',
+    stat.S_IFCHR:'chardev',
+    stat.S_IFBLK:'block',
+    stat.S_IFREG:'file',
+    stat.S_IFIFO:'fifo',
+    stat.S_IFLNK:'symlink',
+    stat.S_IFSOCK:'socket',
+}
+def file_kind(f, _formats=_formats, _unknown='unknown', _lstat=os.lstat):
+    try:
+        return _formats[_lstat(f).st_mode & 0170000]
+    except KeyError:
+        return _unknown
 
 
 def kind_marker(kind):
@@ -104,20 +102,21 @@
     else:
         raise BzrError('invalid file kind %r' % kind)
 
-def lexists(f):
-    if hasattr(os.path, 'lexists'):
-        return os.path.lexists(f)
-    try:
-        if hasattr(os, 'lstat'):
-            os.lstat(f)
-        else:
-            os.stat(f)
-        return True
-    except OSError,e:
-        if e.errno == errno.ENOENT:
-            return False;
-        else:
-            raise BzrError("lstat/stat of (%r): %r" % (f, e))
+lexists = getattr(os.path, 'lexists', None)
+if lexists is None:
+    def lexists(f):
+        try:
+            if hasattr(os, 'lstat'):
+                os.lstat(f)
+            else:
+                os.stat(f)
+            return True
+        except OSError,e:
+            if e.errno == errno.ENOENT:
+                return False;
+            else:
+                raise BzrError("lstat/stat of (%r): %r" % (f, e))
+
 
 def fancy_rename(old, new, rename_func, unlink_func):
     """A fancy rename, when you don't have atomic rename.
@@ -547,6 +546,7 @@
     return pathjoin(*p)
 
 
+@deprecated_function(zero_nine)
 def appendpath(p1, p2):
     if p1 == '':
         return p2

=== modified file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	2006-02-28 04:31:50 +0000
+++ bzrlib/symbol_versioning.py	2006-05-27 01:54:40 +0000
@@ -25,7 +25,7 @@
            'DEPRECATED_PARAMETER',
            'deprecated_passed',
            'warn', 'set_warning_method', 'zero_seven',
-           'zero_eight',
+           'zero_eight', 'zero_nine'
            ]
 
 from warnings import warn
@@ -34,6 +34,7 @@
 DEPRECATED_PARAMETER = "A deprecated parameter marker."
 zero_seven = "%s was deprecated in version 0.7."
 zero_eight = "%s was deprecated in version 0.8."
+zero_nine = "%s was deprecated in version 0.9."
 
 
 def set_warning_method(method):
@@ -113,7 +114,10 @@
 
 def _decorate_docstring(callable, deprecation_version, label,
                         decorated_callable):
-    docstring_lines = callable.__doc__.split('\n')
+    if callable.__doc__:
+        docstring_lines = callable.__doc__.split('\n')
+    else:
+        docstring_lines = []
     if len(docstring_lines) == 0:
         decorated_callable.__doc__ = deprecation_version % ("This " + label)
     elif len(docstring_lines) == 1:

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-05-24 16:16:39 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-05-30 01:15:18 +0000
@@ -62,6 +62,7 @@
                      'bzrlib.tests.blackbox.test_pull',
                      'bzrlib.tests.blackbox.test_push',
                      'bzrlib.tests.blackbox.test_reconcile',
+                     'bzrlib.tests.blackbox.test_remove',
                      'bzrlib.tests.blackbox.test_re_sign',
                      'bzrlib.tests.blackbox.test_revert',
                      'bzrlib.tests.blackbox.test_revno',

=== added file 'bzrlib/tests/blackbox/test_remove.py'
--- bzrlib/tests/blackbox/test_remove.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_remove.py	2006-05-30 02:05:08 +0000
@@ -0,0 +1,62 @@
+# Copyright (C) 2005, 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+
+from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.workingtree import WorkingTree
+
+
+class TestRemove(ExternalBase):
+
+    def test_remove_deleted(self):
+        self.runbzr("init")
+        self.build_tree(['a'])
+        self.runbzr(['add', 'a'])
+        self.runbzr(['commit', '-m', 'added a'])
+        os.unlink('a')
+        self.runbzr(['remove', 'a'])
+
+    def test_remove_new(self):
+        self.build_tree(['filefile',
+                         'dir/',
+                         'dir/filefilefile'])
+        wt = self.make_branch_and_tree('.')
+        wt.add(['filefile', 'dir', 'dir/filefilefile'], 
+               ['filefile-id', 'dir-id', 'filefilefile-id'])
+        self.assertEqual(wt.path2id('filefile'), 'filefile-id')
+        self.assertEqual(wt.path2id('dir/filefilefile'), 'filefilefile-id')
+        self.assertEqual(wt.path2id('dir'), 'dir-id')
+        self.runbzr('remove --new')
+        wt = WorkingTree.open('.')
+        self.assertIs(wt.path2id('filefile'), None)
+        self.assertIs(wt.path2id('dir/filefilefile'), None)
+        self.assertIs(wt.path2id('dir'), None)
+        wt.add(['filefile', 'dir', 'dir/filefilefile'], 
+               ['filefile-id', 'dir-id', 'filefilefile-id'])
+        self.assertEqual(wt.path2id('filefile'), 'filefile-id')
+        self.assertEqual(wt.path2id('dir/filefilefile'), 'filefilefile-id')
+        self.assertEqual(wt.path2id('dir'), 'dir-id')
+        self.runbzr('remove --new dir')
+        wt = WorkingTree.open('.')
+        self.assertEqual(wt.path2id('filefile'), 'filefile-id')
+        self.assertIs(wt.path2id('dir/filefilefile'), None)
+        self.assertIs(wt.path2id('dir'), None)
+        self.runbzr('remove --new .')
+        wt = WorkingTree.open('.')
+        self.assertIs(wt.path2id('filefile'), None)
+        self.runbzr('remove --new .', retcode=3)

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2006-05-24 16:16:39 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2006-05-30 01:15:18 +0000
@@ -103,14 +103,6 @@
         self.runbzr("--pants off", retcode=3)
         self.runbzr("diff --message foo", retcode=3)
 
-    def test_remove_deleted(self):
-        self.runbzr("init")
-        self.build_tree(['a'])
-        self.runbzr(['add', 'a'])
-        self.runbzr(['commit', '-m', 'added a'])
-        os.unlink('a')
-        self.runbzr(['remove', 'a'])
-
     def test_ignore_patterns(self):
         self.runbzr('init')
         self.assertEquals(self.capture('unknowns'), '')

=== modified file 'bzrlib/tests/test_inv.py'
--- bzrlib/tests/test_inv.py	2006-05-19 13:53:22 +0000
+++ bzrlib/tests/test_inv.py	2006-05-27 09:06:37 +0000
@@ -61,6 +61,24 @@
         
         self.assert_('src-id' in inv)
 
+    def test_iter_entries(self):
+        inv = Inventory()
+        
+        for args in [('src', 'directory', 'src-id'), 
+                     ('doc', 'directory', 'doc-id'), 
+                     ('src/hello.c', 'file', 'hello-id'),
+                     ('src/bye.c', 'file', 'bye-id'),
+                     ('Makefile', 'file', 'makefile-id')]:
+            inv.add_path(*args)
+
+        self.assertEqual([
+            ('Makefile', 'makefile-id'),
+            ('doc', 'doc-id'),
+            ('src', 'src-id'),
+            ('src/bye.c', 'bye-id'),
+            ('src/hello.c', 'hello-id'),
+            ], [(path, ie.file_id) for path, ie in inv.iter_entries()])
+            
     def test_version(self):
         """Inventory remembers the text's version."""
         inv = Inventory()

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2006-05-05 01:15:53 +0000
+++ bzrlib/tests/test_osutils.py	2006-05-30 04:33:35 +0000
@@ -17,7 +17,10 @@
 """Tests for the osutils wrapper.
 """
 
+import errno
 import os
+import socket
+import stat
 import sys
 
 import bzrlib
@@ -90,6 +93,40 @@
         self.failIfExists('dir/file')
         self.failIfExists('dir')
 
+    def test_file_kind(self):
+        self.build_tree(['file', 'dir/'])
+        self.assertEquals('file', osutils.file_kind('file'))
+        self.assertEquals('directory', osutils.file_kind('dir/'))
+        if osutils.has_symlinks():
+            os.symlink('symlink', 'symlink')
+            self.assertEquals('symlink', osutils.file_kind('symlink'))
+        
+        # TODO: jam 20060529 Test a block device
+        try:
+            os.lstat('/dev/null')
+        except OSError, e:
+            if e.errno not in (errno.ENOENT,):
+                raise
+        else:
+            self.assertEquals('chardev', osutils.file_kind('/dev/null'))
+
+        mkfifo = getattr(os, 'mkfifo', None)
+        if mkfifo:
+            mkfifo('fifo')
+            try:
+                self.assertEquals('fifo', osutils.file_kind('fifo'))
+            finally:
+                os.remove('fifo')
+
+        AF_UNIX = getattr(socket, 'AF_UNIX', None)
+        if AF_UNIX:
+            s = socket.socket(AF_UNIX)
+            s.bind('socket')
+            try:
+                self.assertEquals('socket', osutils.file_kind('socket'))
+            finally:
+                os.remove('socket')
+
 
 class TestSafeUnicode(TestCase):
 

=== modified file 'bzrlib/tests/test_xml.py'
--- bzrlib/tests/test_xml.py	2006-04-06 10:21:12 +0000
+++ bzrlib/tests/test_xml.py	2006-05-28 03:56:21 +0000
@@ -19,7 +19,7 @@
 from bzrlib.tests import TestCase
 from bzrlib.inventory import Inventory, InventoryEntry
 from bzrlib.xml4 import serializer_v4
-from bzrlib.xml5 import serializer_v5
+import bzrlib.xml5
 
 _working_inventory_v4 = """<inventory file_id="TREE_ROOT">
 <entry file_id="bar-20050901064931-73b4b1138abc9cd2" kind="file" name="bar" parent_id="TREE_ROOT" />
@@ -110,7 +110,7 @@
     def test_unpack_revision_5(self):
         """Test unpacking a canned revision v5"""
         inp = StringIO(_revision_v5)
-        rev = serializer_v5.read_revision(inp)
+        rev = bzrlib.xml5.serializer_v5.read_revision(inp)
         eq = self.assertEqual
         eq(rev.committer,
            "Martin Pool <mbp@sourcefrog.net>")
@@ -122,7 +122,7 @@
     def test_unpack_inventory_5(self):
         """Unpack canned new-style inventory"""
         inp = StringIO(_committed_inv_v5)
-        inv = serializer_v5.read_inventory(inp)
+        inv = bzrlib.xml5.serializer_v5.read_inventory(inp)
         eq = self.assertEqual
         eq(len(inv), 4)
         ie = inv['bar-20050824000535-6bc48cfad47ed134']
@@ -134,7 +134,7 @@
     def test_unpack_basis_inventory_5(self):
         """Unpack canned new-style inventory"""
         inp = StringIO(_basis_inv_v5)
-        inv = serializer_v5.read_inventory(inp)
+        inv = bzrlib.xml5.serializer_v5.read_inventory(inp)
         eq = self.assertEqual
         eq(len(inv), 4)
         eq(inv.revision_id, 'mbp@sourcefrog.net-20050905063503-43948f59fa127d92')
@@ -146,28 +146,28 @@
 
     def test_repack_inventory_5(self):
         inp = StringIO(_committed_inv_v5)
-        inv = serializer_v5.read_inventory(inp)
+        inv = bzrlib.xml5.serializer_v5.read_inventory(inp)
         outp = StringIO()
-        serializer_v5.write_inventory(inv, outp)
-        inv2 = serializer_v5.read_inventory(StringIO(outp.getvalue()))
+        bzrlib.xml5.serializer_v5.write_inventory(inv, outp)
+        inv2 = bzrlib.xml5.serializer_v5.read_inventory(StringIO(outp.getvalue()))
         self.assertEqual(inv, inv2)
 
     def test_repack_revision_5(self):
         """Round-trip revision to XML v5"""
         inp = StringIO(_revision_v5)
-        rev = serializer_v5.read_revision(inp)
+        rev = bzrlib.xml5.serializer_v5.read_revision(inp)
         outp = StringIO()
-        serializer_v5.write_revision(rev, outp)
+        bzrlib.xml5.serializer_v5.write_revision(rev, outp)
         outfile_contents = outp.getvalue()
-        rev2 = serializer_v5.read_revision(StringIO(outfile_contents))
+        rev2 = bzrlib.xml5.serializer_v5.read_revision(StringIO(outfile_contents))
         self.assertEqual(rev, rev2)
 
     def test_pack_revision_5(self):
         """Pack revision to XML v5"""
         # fixed 20051025, revisions should have final newline
-        rev = serializer_v5.read_revision_from_string(_revision_v5)
+        rev = bzrlib.xml5.serializer_v5.read_revision_from_string(_revision_v5)
         outp = StringIO()
-        serializer_v5.write_revision(rev, outp)
+        bzrlib.xml5.serializer_v5.write_revision(rev, outp)
         outfile_contents = outp.getvalue()
         self.assertEqual(outfile_contents[-1], '\n')
-        self.assertEqualDiff(outfile_contents, serializer_v5.write_revision_to_string(rev))
+        self.assertEqualDiff(outfile_contents, bzrlib.xml5.serializer_v5.write_revision_to_string(rev))

=== modified file 'bzrlib/tests/workingtree_implementations/test_basis_inventory.py'
--- bzrlib/tests/workingtree_implementations/test_basis_inventory.py	2006-04-18 22:41:16 +0000
+++ bzrlib/tests/workingtree_implementations/test_basis_inventory.py	2006-05-28 03:56:21 +0000
@@ -18,7 +18,7 @@
 
 from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
 from bzrlib.branch import Branch
-from bzrlib.xml5 import serializer_v5
+import bzrlib.xml5
 
 
 class TestBasisInventory(TestCaseWithWorkingTree):
@@ -49,7 +49,7 @@
         t._control_files.get_utf8('basis-inventory')
 
         basis_inv_txt = t.read_basis_inventory()
-        basis_inv = serializer_v5.read_inventory_from_string(basis_inv_txt)
+        basis_inv = bzrlib.xml5.serializer_v5.read_inventory_from_string(basis_inv_txt)
         self.assertEquals('r2', basis_inv.revision_id)
         store_inv = b.repository.get_inventory('r2')
 

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-05-18 23:48:54 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-05-27 09:24:05 +0000
@@ -37,10 +37,9 @@
 
 class TestWorkingTree(TestCaseWithWorkingTree):
 
-    def test_listfiles(self):
+    def test_list_files(self):
         tree = self.make_branch_and_tree('.')
-        os.mkdir('dir')
-        print >> open('file', 'w'), "content"
+        self.build_tree(['dir/', 'file'])
         if has_symlinks():
             os.symlink('target', 'symlink')
         files = list(tree.list_files())
@@ -49,6 +48,32 @@
         if has_symlinks():
             self.assertEqual(files[2], ('symlink', '?', 'symlink', None, TreeLink()))
 
+    def test_list_files_sorted(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['dir/', 'file', 'dir/file', 'dir/b', 'dir/subdir/', 'a', 'dir/subfile',
+                'zz_dir/', 'zz_dir/subfile'])
+        files = [(path, kind) for (path, versioned, kind, file_id, entry) in tree.list_files()]
+        self.assertEqual([
+            ('a', 'file'),
+            ('dir', 'directory'),
+            ('file', 'file'),
+            ('zz_dir', 'directory'),
+            ], files)
+
+        tree.add(['dir', 'zz_dir'])
+        files = [(path, kind) for (path, versioned, kind, file_id, entry) in tree.list_files()]
+        self.assertEqual([
+            ('a', 'file'),
+            ('dir', 'directory'),
+            ('dir/b', 'file'),
+            ('dir/file', 'file'),
+            ('dir/subdir', 'directory'),
+            ('dir/subfile', 'file'),
+            ('file', 'file'),
+            ('zz_dir', 'directory'),
+            ('zz_dir/subfile', 'file'),
+            ], files)
+
     def test_open_containing(self):
         branch = self.make_branch_and_tree('.').branch
         wt, relpath = WorkingTree.open_containing()

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2006-05-12 00:55:30 +0000
+++ bzrlib/tree.py	2006-06-03 16:23:09 +0000
@@ -24,7 +24,7 @@
 from bzrlib.trace import mutter, note
 from bzrlib.errors import BzrError, BzrCheckError
 from bzrlib.inventory import Inventory
-from bzrlib.osutils import appendpath, fingerprint_file
+from bzrlib.osutils import fingerprint_file
 
 class Tree(object):
     """Abstract file tree.
@@ -161,12 +161,13 @@
     def get_file_size(self, file_id):
         return self._inventory[file_id].text_size
 
-    def get_file_sha1(self, file_id):
+    def get_file_sha1(self, file_id, path=None):
         ie = self._inventory[file_id]
         if ie.kind == "file":
             return ie.text_sha1
+        return None
 
-    def is_executable(self, file_id):
+    def is_executable(self, file_id, path=None):
         ie = self._inventory[file_id]
         if ie.kind != "file":
             return None 
@@ -214,7 +215,7 @@
     def __contains__(self, file_id):
         return file_id in self._inventory
 
-    def get_file_sha1(self, file_id):
+    def get_file_sha1(self, file_id, path=None):
         assert self._inventory[file_id].kind == "root_directory"
         return None
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-05-19 07:57:09 +0000
+++ bzrlib/workingtree.py	2006-05-27 09:24:05 +0000
@@ -40,6 +40,7 @@
 # memory.  (Now done? -- mbp 20060309)
 
 from binascii import hexlify
+import collections
 from copy import deepcopy
 from cStringIO import StringIO
 import errno
@@ -73,7 +74,6 @@
 from bzrlib.merge import merge_inner, transform_tree
 from bzrlib.osutils import (
                             abspath,
-                            appendpath,
                             compact_date,
                             file_kind,
                             isdir,
@@ -283,7 +283,7 @@
         hc = self._hashcache = HashCache(basedir, cache_filename, self._control_files._file_mode)
         hc.read()
         # is this scan needed ? it makes things kinda slow.
-        hc.scan()
+        #hc.scan()
 
         if hc.needs_write:
             mutter("write hc")
@@ -530,15 +530,18 @@
         return os.path.getsize(self.id2abspath(file_id))
 
     @needs_read_lock
-    def get_file_sha1(self, file_id):
-        path = self._inventory.id2path(file_id)
+    def get_file_sha1(self, file_id, path=None):
+        if not path:
+            path = self._inventory.id2path(file_id)
         return self._hashcache.get_sha1(path)
 
-    def is_executable(self, file_id):
-        if not supports_executable():
+    if not supports_executable():
+        def is_executable(self, file_id, path=None):
             return self._inventory[file_id].executable
-        else:
-            path = self._inventory.id2path(file_id)
+    else:
+        def is_executable(self, file_id, path=None):
+            if not path:
+                path = self._inventory.id2path(file_id)
             mode = os.lstat(self.abspath(path)).st_mode
             return bool(stat.S_ISREG(mode) and stat.S_IEXEC&mode)
 
@@ -690,7 +693,7 @@
             return '?'
 
     def list_files(self):
-        """Recursively list all files as (path, class, kind, id).
+        """Recursively list all files as (path, class, kind, id, entry).
 
         Lists, but does not descend into unversioned directories.
 
@@ -700,29 +703,50 @@
         Skips the control directory.
         """
         inv = self._inventory
-
-        def descend(from_dir_relpath, from_dir_id, dp):
-            ls = os.listdir(dp)
-            ls.sort()
-            for f in ls:
+        # Convert these into local objects to save lookup times
+        pathjoin = bzrlib.osutils.pathjoin
+        file_kind = bzrlib.osutils.file_kind
+
+        # transport.base ends in a slash, we want the piece
+        # between the last two slashes
+        transport_base_dir = self.bzrdir.transport.base.rsplit('/', 2)[1]
+
+        fk_entries = {'directory':TreeDirectory, 'file':TreeFile, 'symlink':TreeLink}
+
+        # directory file_id, relative path, absolute path, reverse sorted children
+        children = os.listdir(self.basedir)
+        children.sort()
+        # jam 20060527 The kernel sized tree seems equivalent whether we 
+        # use a deque and popleft to keep them sorted, or if we use a plain
+        # list and just reverse() them.
+        children = collections.deque(children)
+        stack = [(inv.root.file_id, u'', self.basedir, children)]
+        while stack:
+            from_dir_id, from_dir_relpath, from_dir_abspath, children = stack[-1]
+
+            while children:
+                f = children.popleft()
                 ## TODO: If we find a subdirectory with its own .bzr
                 ## directory, then that is a separate tree and we
                 ## should exclude it.
 
                 # the bzrdir for this tree
-                if self.bzrdir.transport.base.endswith(f + '/'):
+                if transport_base_dir == f:
                     continue
 
-                # path within tree
-                fp = appendpath(from_dir_relpath, f)
+                # we know that from_dir_relpath and from_dir_abspath never end in a slash
+                # and 'f' doesn't begin with one, we can do a string op, rather
+                # than the checks of pathjoin(), all relative paths will have an extra slash
+                # at the beginning
+                fp = from_dir_relpath + '/' + f
 
                 # absolute path
-                fap = appendpath(dp, f)
+                fap = from_dir_abspath + '/' + f
                 
                 f_ie = inv.get_child(from_dir_id, f)
                 if f_ie:
                     c = 'V'
-                elif self.is_ignored(fp):
+                elif self.is_ignored(fp[1:]):
                     c = 'I'
                 else:
                     c = '?'
@@ -737,31 +761,28 @@
 
                 # make a last minute entry
                 if f_ie:
-                    entry = f_ie
+                    yield fp[1:], c, fk, f_ie.file_id, f_ie
                 else:
-                    if fk == 'directory':
-                        entry = TreeDirectory()
-                    elif fk == 'file':
-                        entry = TreeFile()
-                    elif fk == 'symlink':
-                        entry = TreeLink()
-                    else:
-                        entry = TreeEntry()
+                    try:
+                        yield fp[1:], c, fk, None, fk_entries[fk]()
+                    except KeyError:
+                        yield fp[1:], c, fk, None, TreeEntry()
+                    continue
                 
-                yield fp, c, fk, (f_ie and f_ie.file_id), entry
-
                 if fk != 'directory':
                     continue
 
-                if c != 'V':
-                    # don't descend unversioned directories
-                    continue
-                
-                for ff in descend(fp, f_ie.file_id, fap):
-                    yield ff
+                # But do this child first
+                new_children = os.listdir(fap)
+                new_children.sort()
+                new_children = collections.deque(new_children)
+                stack.append((f_ie.file_id, fp, fap, new_children))
+                # Break out of inner loop, so that we start outer loop with child
+                break
+            else:
+                # if we finished all children, pop it off the stack
+                stack.pop()
 
-        for f in descend(u'', inv.root.file_id, self.basedir):
-            yield f
 
     @needs_write_lock
     def move(self, from_paths, to_name):
@@ -803,7 +824,7 @@
             if f_id == None:
                 raise BzrError("%r is not versioned" % f)
             name_tail = splitpath(f)[-1]
-            dest_path = appendpath(to_name, name_tail)
+            dest_path = pathjoin(to_name, name_tail)
             if self.has_filename(dest_path):
                 raise BzrError("destination %r already exists" % dest_path)
             if f_id in to_idpath:
@@ -816,7 +837,7 @@
         try:
             for f in from_paths:
                 name_tail = splitpath(f)[-1]
-                dest_path = appendpath(to_name, name_tail)
+                dest_path = pathjoin(to_name, name_tail)
                 result.append((f, dest_path))
                 inv.rename(inv.path2id(f), to_dir_id, name_tail)
                 try:
@@ -912,7 +933,8 @@
 
     def _iter_conflicts(self):
         conflicted = set()
-        for path in (s[0] for s in self.list_files()):
+        for info in self.list_files():
+            path = info[0]
             stem = get_conflicted_stem(path)
             if stem is None:
                 continue
@@ -978,7 +1000,7 @@
             
             fl.sort()
             for subf in fl:
-                subp = appendpath(path, subf)
+                subp = pathjoin(path, subf)
                 yield subp
 
     def _translate_ignore_rule(self, rule):

